import unittest
from os import path
from itertools import combinations
from pyimagetest import ImageTestcase


class Tester(ImageTestcase, unittest.TestCase):
    @property
    def default_test_image_file(self) -> str:
        # The test image was downloaded from
        # http://www.r0k.us/graphics/kodak/kodim15.html
        # and is cleared for unrestricted usage
        return path.join(path.dirname(__file__), "test_image.png")

    def test_io(self) -> None:
        for backend1, backend2 in combinations(self.backends.values(), 2):
            image1 = self.load_image(backend1)
            image2 = self.load_image(backend2)
            self.assertImagesAlmostEqual(
                image1, image2, image1_backend=backend1, image2_backend=backend2
            )


if __name__ == "__main__":
    unittest.main()



from abc import ABC, abstractmethod
from typing import Any, Union, Optional
from collections import OrderedDict
import numpy as np
from .backend import ImageBackend, builtin_image_backends

__all__ = ["ImageTestcase"]


class ImageTestcase(ABC):
    """Utility class for unit testing with images. This class is meant for double
    inheritance together with unittest.Testcase.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.backends = OrderedDict()
        self._add_builtin_image_backends()

    def add_image_backend(self, name: str, backend: ImageBackend) -> None:
        """Adds custom image backend to the list of available backends.

        Args:
            name (str): Name of the backend
            backend (ImageBackend): Backend
        """
        self.backends[name] = backend

    def _add_builtin_image_backends(self) -> None:
        self.backends.update(builtin_image_backends())

    @property
    @abstractmethod
    def default_test_image_file(self) -> str:
        """Returns the path to the default test image file
        Returns:
            str
        """
        pass

    def load_image(
        self, backend: Union[ImageBackend, str], file: Optional[str] = None
    ) -> Any:
        """Loads an image with the given image backend. If no file is given, the
        default test image is loaded.

        Args:
            backend (Union[ImageBackend, str]): Backend or backend name
            file (Optional[str]): Path to image file. If None is given, the default
            test image is used.

        Returns:
            Any
        """
        if isinstance(backend, str):
            backend = self.backends[backend]
        if file is None:
            file = self.default_test_image_file
        return backend.import_image(file)

    def assertImagesAlmostEqual(
        self,
        image1: Any,
        image2: Any,
        mean_abs_tolerance: float = 1e-2,
        image1_backend: Optional[ImageBackend] = None,
        image2_backend: Optional[ImageBackend] = None,
    ):
        """This test verifies that the two images are almost equal.

        Args:
            image1 (Any): Image 1
            image2 (Any): Image 2
            mean_abs_tolerance: Acceptable mean absolute tolerance (MAE)
            image1_backend (Union[ImageBackend, str]): Backend or backend name for
                image 1. If None, the backend is inferred automatically from the image.
            image2_backend (Union[ImageBackend, str]): Backend or backend name for
                image 2. If None, the backend is inferred automatically from the image.
        """
        if isinstance(image1_backend, str):
            image1_backend = self.backends[image1_backend]
        elif image1_backend is None:
            image1_backend = self.infer_image_backend(image1)

        if isinstance(image2_backend, str):
            image2_backend = self.backends[image2_backend]
        elif image2_backend is None:
            image2_backend = self.infer_image_backend(image2)

        image1 = image1_backend.export_image(image1)
        image2 = image2_backend.export_image(image2)

        actual = np.mean(np.abs(image1 - image2))
        desired = 0.0
        np.testing.assert_allclose(actual, desired, atol=mean_abs_tolerance, rtol=0.0)

    def infer_image_backend(self, image: Any) -> str:
        """Infers the corresponding backend from the image.

        Args:
            image: Image with type of any known backend

        Returns:
            ImageBackend
        """
        for backend in self.backends.values():
            if image in backend:
                return backend
        raise RuntimeError



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

install_requires = ("numpy",)

classifiers = (
    "Development Status :: 3 - Alpha",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 3",
    "Topic :: Software Development :: Testing :: Unit",
)

setup(
    name="pyimagetest",
    description="Utilities for unit testing with images.",
    version="0.1",
    url="https://github.com/pmeier/pyimagetest",
    license="BSD-3",
    author="Philip Meier",
    author_email="github.pmeier@posteo.de",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=("test",)),
    install_requires=install_requires,
    python_requires=">=3.6",
    classifiers=classifiers,
)



from abc import ABC, abstractmethod
from typing import Type, Union
from collections import OrderedDict
import numpy as np

try:
    import imageio

    imageioImageType = np.ndarray
except ImportError:
    imageio = None
    imageioImageType = None

try:
    from PIL import Image

    PILImageType = Image.Image
except ImportError:
    Image = None
    PILImageType = None

ImageType = Union[imageioImageType, PILImageType]

__all__ = ["ImageBackend", "builtin_image_backends"]


class ImageBackend(ABC):
    """ABC for ImageBackends. Each subclass has to implement the native ImageType and
    basic I/O methods.
    """

    @property
    @abstractmethod
    def native_image_type(self) -> Type[ImageType]:
        """Returns the native ImageType of the backend. This is used to infer the
        specific `ImageBackend` for a given `ImageType`

        Returns:
            Type[ImageType]
        """
        pass

    def __contains__(self, image) -> bool:
        return isinstance(image, self.native_image_type)

    @abstractmethod
    def import_image(self, file: str) -> ImageType:
        """Imports an image into the specific image type of the backend.

        Args:
            file (str): Path to the file that should be imported.

        Returns:
            ImageType
        """
        pass

    @abstractmethod
    def export_image(self, image: ImageType) -> np.ndarray:
        """Exports an image of the specific image type of the backend into a
        numpy.ndarray. The array should be of size height x width x channels (HxWxC) and
        be of type np.float32.

        Args:
            image (ImageType): Image object

        Returns:
            np.ndarray
        """
        pass


class ImageioBackend(ImageBackend):
    """
    `ImageBackend for the `imageio <https://imageio.github.io/>`_ package.
    """

    @property
    def native_image_type(self) -> Type[imageioImageType]:
        return imageioImageType

    def import_image(self, file: str) -> imageioImageType:
        return imageio.imread(file)

    def export_image(self, image: imageioImageType) -> np.ndarray:
        return image.astype(np.float32) / 255.0


class PILBackend(ImageBackend):
    """
    `ImageBackend for the `PIL (Pillow) <https://python-pillow.org/`_ package.
    """

    @property
    def native_image_type(self) -> Type[PILImageType]:
        return PILImageType

    def import_image(self, file: str) -> PILImageType:
        return Image.open(file)

    def export_image(self, image: PILImageType) -> np.ndarray:
        mode = image.mode
        image = np.asarray(image, dtype=np.float32)
        if mode in ("L", "RGB"):
            image /= 255.0
        if mode in ("1", "L"):
            image = np.expand_dims(image, 2)
        return image


BUILTIN_IMAGE_BACKENDS = OrderedDict(
    [
        ("imageio", ImageioBackend() if imageioImageType is not None else None),
        ("PIL", PILBackend() if PILImageType is not None else None),
    ]
)


def builtin_image_backends():
    """Returns all builtin image backends, which are available.

    Returns:
        OrderedDict[str, Backend]
    """
    return OrderedDict(
        [
            (name, backend)
            for name, backend in BUILTIN_IMAGE_BACKENDS.items()
            if backend is not None
        ]
    )
